"""Napr package."""

from napr import apps, data, plotting, utils, evaluation, hyperopt

__version__ = "0.1.5"
__author__ = "Morteza Hosseini"
__license__ = "MIT"

__all__ = ["apps", "data", "plotting", "utils", "evaluation", "hyperopt"]
